/************************************************************************/
/* shutter_example.c                                                    */
/*                                                                      */
/* Example on how to talk to the shutters.				*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE                        */
/*                 Based on the SUSI User Interface                     */
/*              In turn based on the CHIP User interface                */
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA                 */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu                                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : Oct 2001 			                                */
/************************************************************************/

#include <cliserv.h>
#include "../../../libs/include/chara_messages.h"
#include <shutter.h>
#include <time.h>
#include <string.h>

/* Prototypes */

int setup_messages(void);
int message_shutter_status(int server, struct smessage *mess);
int message_shutter_conf(int server, struct smessage *mess);
int send_shutter_open(unsigned char shutter);
int send_shutter_close(unsigned char shutter);
int send_shutter_status(unsigned char shutter);
int shutter_id(char *name);
int message_noop(int server, struct smessage *mess);
int message_message(int server, struct smessage *mess);
int message_ping(int server, struct smessage *mess);
int message_server_stopping(int server, struct smessage *mess);

/* globals */

int shutter_server = -1;
struct sshutter_conf shutter_conf;

int main(int   argc, char *argv[])
{
        struct smessage mess;
	time_t	 start;
	int	irb3;

	/* Initialise mesage jobs */

	init_jobs();

	/* Open the sockets */

        if ((shutter_server = open_server_by_name("shutters")) == -1)
        {
                fprintf(stderr,"Failed to open process shutters");
		exit(-1);
        }

        /* Setup up local message servers */

        if (!setup_messages()) 
        {
                fprintf(stderr,"Could not setup messages.");
        }

	/* Send out requests for machine name and port names */

	*(shutter_conf.name) = 0;
        mess.type = SHUTTER_CONF;
        mess.length = 0;
        mess.data = NULL;

        if (!send_message(shutter_server, &mess))
	{
                fprintf(stderr,"Failed to send SHUTTER_CONF.");
		exit(-3);
	}

	/* Now let's wait a while and see if we get an answer. */

	start = time(NULL);
	while(time(NULL) < start+TIME_OUT && *(shutter_conf.name) == 0)
	{
		process_server_socket(shutter_server);
	}

	if (*(shutter_conf.name) == 0)
	{
		fprintf(stderr,"Timed out waiting for response from server.\n");
		exit(-7);
	}

	/* Let's open and then close one */

	irb3 = shutter_id("IRB3");
	send_shutter_open(irb3);
	sleep(1);
	send_shutter_close(irb3);

	/* PING! */

        mess.type = PING;
        mess.length = 0;
        mess.data = NULL;

        if (!send_message(shutter_server, &mess))
	{
                fprintf(stderr,"Failed to send PING.");
		exit(-3);
	}

	/* You need to call this function regularly */

	while(1) process_server_socket(shutter_server);

	return(0);
}

/************************************************************************/
/* setup_messages()							*/
/*									*/
/* Guess what this does.						*/
/************************************************************************/

int setup_messages(void)
{
        if (!add_message_job(shutter_server, NOOP, message_noop)) return FALSE;
        if (!add_message_job(shutter_server, MESSAGE, message_message)) return FALSE;
        if (!add_message_job(shutter_server, PING, message_ping)) return FALSE;
        if (!add_message_job(shutter_server, SERVER_STOPPING, 
		message_server_stopping)) return FALSE;
        if (!add_message_job(shutter_server, STATUS_SHUTTER, message_shutter_status)) 
		return FALSE;
        if (!add_message_job(shutter_server, SHUTTER_CONF, message_shutter_conf)) 
		return FALSE;

	return TRUE;

} /* setup_messages() */

/************************************************************************/
/* message_shutter_status()						*/
/*									*/
/************************************************************************/

int message_shutter_status(int server, struct smessage *mess)
{
	struct sshutter_status *status;

        if (mess->length != sizeof(*status))
        {
		fprintf(stderr,
			"Receiced STATUS_SHUTTER with wrong amount of data.");
		return FALSE;
        }

	status = (struct sshutter_status *)mess->data;

	printf("Received shutter status\n");

	if (status->status == INSANE)
	{
		fprintf(stderr,
			"ERROR\nShutter %s is reported as INSANE!",
			shutter_conf.shutter_names[status->shutter]);
	}

        return TRUE;

} /* message_shutter_status() */

/************************************************************************/
/* message_shutter_conf()						*/
/*									*/
/************************************************************************/

int message_shutter_conf(int server, struct smessage *mess)
{
	int	i;
	unsigned char	*p1, *p2;

	/* OK, we just copy the thing we got in the message */

	if (mess->length != sizeof(shutter_conf))
	{
	    fprintf(stderr,"Wrong number of bytes in configuration.");
	    return FALSE;
	}

	printf("Received shutter configuration\n");

	for(i=0, p1 = (unsigned char *)&shutter_conf, p2 = mess->data;
		i < sizeof(shutter_conf); i++, p1++, p2++) *p1 = *p2;

	return TRUE;

} /* message_shutter_conf() */

/************************************************************************/
/* send_shutter_open()							*/
/* 									*/
/* Send the OPEN_SHUTTER message to the server.				*/
/* Returns TRUE or FALSE.						*/
/************************************************************************/

int send_shutter_open(unsigned char shutter)
{
	struct smessage	mess;

	/* Make sure everything is OK */

	if (!server_socket_open(shutter_server)) return FALSE;

	if (shutter >= NUMBER_SHUTTERS) return FALSE;

	/* Construct command */

	mess.type = OPEN_SHUTTER;
	mess.length = sizeof(shutter);
	mess.data = &shutter;

	if (!send_message(shutter_server, &mess))
		fprintf(stderr,"Failed to send OPEN_SHUTTER message.");

	return TRUE;

} /* send_shutter_open() */

/************************************************************************/
/* send_shutter_close()							*/
/* 									*/
/* Send the CLOSE_SHUTTER message to the server.			*/
/* Returns an TRUE or FALSE.						*/
/************************************************************************/

int send_shutter_close(unsigned char shutter)
{
	struct smessage	mess;

	/* Make sure everything is OK */

	if (!server_socket_open(shutter_server)) return FALSE;

	if (shutter >= NUMBER_SHUTTERS) return FALSE;

	/* Construct command */

	mess.type = CLOSE_SHUTTER;
	mess.length = sizeof(shutter);
	mess.data = &shutter;

	if (!send_message(shutter_server, &mess))
		fprintf(stderr,"Failed to send CLOSE_SHUTTER message.");

	return TRUE;

} /* send_shutter_close() */

/************************************************************************/
/* send_shutter_status()						*/
/* 									*/
/* Send the STATUS_SHUTTER message to the server.			*/
/* Returns an TRUE or FALSE.						*/
/************************************************************************/

int send_shutter_status(unsigned char shutter)
{
	struct smessage	mess;
	struct sshutter_status status;

	/* Make sure everything is OK */

	if (!server_socket_open(shutter_server)) return FALSE;

	if (shutter >= NUMBER_SHUTTERS) return FALSE;

	/* Construct command */

	status.shutter = shutter;
	status.status  = 0;
	mess.type = STATUS_SHUTTER;
	mess.length = sizeof(status);
	mess.data = (unsigned char *)&status;

	if (!send_message(shutter_server, &mess))
		fprintf(stderr,"Failed to send STATUS_SHUTTER message.");

	return TRUE;

} /* send_shutter_status() */

/************************************************************************/
/* shutter_id()                                                         */
/*                                                                      */
/* Given the string name for a shutter as an argument returns the       */
/* shutter number for that shutter, or -1 if no name like that is       */
/* found.                                                               */
/************************************************************************/

int shutter_id(char *name)
{
        int     id;

        for (id=0; id<NUMBER_SHUTTERS; id++)
        {
                if (strcmp(name,shutter_conf.shutter_names[id]) == 0)
                {
                        return id;
                }
        }

        return -1;

} /* shutter_id() */

/************************************************************************/
/* message_noop()							*/
/*									*/
/* A message to do nothing.						*/
/************************************************************************/

int message_noop(int server, struct smessage *mess)
{
	printf("\nI just received a NOOP message.\n");
	return TRUE;
}

/************************************************************************/
/* message_message()							*/
/*									*/
/* A message to display a message.					*/
/************************************************************************/

int message_message(int server, struct smessage *mess)
{
        char *s,*p1,*p2;
        int     i;

        if (mess->length == 0)
        {
		return TRUE;
        }
        else
        {
                if ((s = malloc(mess->length+1)) == NULL)
                {
			fprintf(stderr,
				"Ran out of memory getting message string.\n");
			exit(-7);
                }

                for(i=0, p1=s, p2=(char *)mess->data;
                    i < mess->length; i++, p1++, p2++) *p1 = *p2;
                *p1 = 0;

		printf("\nMessage from server: %s\n", s);

                free(s);
        }

	return TRUE;

} /* message_message() */

/************************************************************************/
/* message_ping()							*/
/*									*/
/* If we get pinged by server, although why I do not know.		*/
/************************************************************************/

int message_ping(int server, struct smessage *mess)
{
        fprintf(stderr, "Example Client is alive.");
	return TRUE;

} /* message_ping() */

/************************************************************************/
/* message_server_stopping()						*/
/*									*/
/* What do we do if we get told the server has died?			*/
/************************************************************************/

int message_server_stopping(int server, struct smessage *mess)
{
	printf("\n\nServer is stopping.\n");
	close_server_socket(server);
	server = -1;
	return TRUE;

} /* message_server_stopping() */
